@@ -43,6 +43,7 @@ RESOURCE_ROOT = PROJECT_ROOT / "resources"
         (io.BytesIO(b"  "), True),
         (io.BytesIO(b"  \n"), True),
         (io.BytesIO(b"    \n"), True),
+        (io.BytesIO(b"\f"), True),
     ],
 )
 def test_skip_over_whitespace(stream, expected):
@@ -60,6 +61,7 @@ def test_skip_over_whitespace(stream, expected):
         (b"  ", True),
         (b"  \n", True),
         (b"    \n", True),
+        (b"\f", True),
     ],
 )
 def test_check_if_whitespace_only(value, expected):
@@ -109,22 +111,6 @@ def test_mark_location():
     Path("pypdf_pdfLocation.txt").unlink()  # cleanup
 
 
-@pytest.mark.parametrize(
-    ("input_str", "expected"),
-    [
-        ("foo", b"foo"),
-        ("😀", "😀".encode()),
-        ("‰", "‰".encode()),
-        ("▷", "▷".encode()),
-        ("世", "世".encode()),
-        # A multi-character string example with non-latin-1 characters:
-        ("😀😃", "😀😃".encode()),
-    ],
-)
-def test_b(input_str: str, expected: bytes):
-    assert pypdf._utils.b_(input_str) == expected
-
-
 def test_deprecate_no_replacement():
     with pytest.warns(DeprecationWarning) as warn:
         pypdf._utils.deprecate_no_replacement("foo", removed_in="3.0.0")
@@ -132,24 +118,6 @@ def test_deprecate_no_replacement():
     assert warn[0].message.args[0] == error_msg
 
 
-@pytest.mark.parametrize(
-    ("left", "up", "upleft", "expected"),
-    [
-        (0, 0, 0, 0),
-        (1, 0, 0, 1),
-        (0, 1, 0, 1),
-        (0, 0, 1, 0),
-        (1, 2, 3, 1),
-        (2, 1, 3, 1),
-        (1, 3, 2, 2),
-        (3, 1, 2, 2),
-        (3, 2, 1, 3),
-    ],
-)
-def test_paeth_predictor(left, up, upleft, expected):
-    assert pypdf._utils.paeth_predictor(left, up, upleft) == expected
-
-
 @pytest.mark.parametrize(
     ("dat", "pos", "to_read", "expected", "expected_pos"),
     [
@@ -234,7 +202,7 @@ def test_read_previous_line2():
 def test_get_max_pdf_version_header():
     with pytest.raises(ValueError) as exc:
         _get_max_pdf_version_header(b"", b"PDF-1.2")
-    assert exc.value.args[0] == "neither b'' nor b'PDF-1.2' are proper headers"
+    assert exc.value.args[0] == "Neither b'' nor b'PDF-1.2' are proper headers"
 
 
 def test_read_block_backwards_exception():
@@ -387,22 +355,22 @@ def test_is_sublist():
         ("1", "1", False),
         ("1.0", "1.1", True),
         ("1", "1.1", True),
-        # suffix left
+        # Suffix left
         ("1a", "2", True),
         ("2a", "1", False),
         ("1a", "1", False),
         ("1.0a", "1.1", True),
-        # I'm not sure about that, but seems special enoguht that it
+        # I'm not sure about that, but seems special enough that it
         # probably doesn't matter:
         ("1a", "1.1", False),
-        # suffix right
+        # Suffix right
         ("1", "2a", True),
         ("2", "1a", False),
         ("1", "1a", True),
         ("1.0", "1.1a", True),
         ("1", "1.1a", True),
         ("", "0.0.0", True),
-        # just suffix matters ... hm, I think this is actually wrong:
+        # Just suffix matters ... hm, I think this is actually wrong:
         ("1.0a", "1.0", False),
         ("1.0", "1.0a", True),
     ],
@@ -413,13 +381,13 @@ def test_version_compare(left, right, is_less_than):
 
 def test_version_compare_equal_str():
     a = Version("1.0")
-    assert (a == "1.0") is False
+    assert a != "1.0"
 
 
 def test_version_compare_lt_str():
     a = Version("1.0")
     with pytest.raises(ValueError) as exc:
-        a < "1.0"  # noqa
+        a < "1.0"  # noqa: B015
     assert exc.value.args[0] == "Version cannot be compared against <class 'str'>"
 
 
